Package: cdata
Type: Package
Title: Fluid Data Transformations
Version: 1.2.1
Date: 2023-08-19
Authors@R: c(
    person("John", "Mount", email = "jmount@win-vector.com", role = c("aut", "cre")),
    person("Nina", "Zumel", email = "nzumel@win-vector.com", role = c("aut")),
    person(family = "Win-Vector LLC", role = c("cph"))
    )
URL: https://github.com/WinVector/cdata/,
        https://winvector.github.io/cdata/
Maintainer: John Mount <jmount@win-vector.com>
BugReports: https://github.com/WinVector/cdata/issues
Description: Supplies higher-order coordinatized data specification and fluid transform operators that include pivot and anti-pivot as special cases. 
    The methodology is describe in 'Zumel', 2018, "Fluid data reshaping with 'cdata'", <https://winvector.github.io/FluidData/FluidDataReshapingWithCdata.html> , <DOI:10.5281/zenodo.1173299> .
    This package introduces the idea of explicit control table specification of data transforms.
    Works on in-memory data or on remote data using 'rquery' and 'SQL' database interfaces.
License: GPL-2 | GPL-3
Encoding: UTF-8
RoxygenNote: 7.2.3
Depends: R (>= 3.4.0), wrapr (>= 2.0.9)
Imports: rquery (>= 1.4.9), rqdatatable (>= 1.3.2), methods, stats
Suggests: DBI, RSQLite, knitr, rmarkdown, yaml, tinytest
VignetteBuilder: knitr
ByteCompile: true
NeedsCompilation: no
Packaged: 2023-08-19 23:37:26 UTC; johnmount
Author: John Mount [aut, cre],
  Nina Zumel [aut],
  Win-Vector LLC [cph]
Repository: CRAN
Date/Publication: 2023-08-20 00:02:32 UTC
Built: R 4.6.0; ; 2026-04-28 00:26:20 UTC; unix
