# Generated by roxygen2: do not edit by hand

S3method(apply_right,blocks_to_rowrecs_spec)
S3method(apply_right,cdata_general_transform_spec)
S3method(apply_right,rowrecs_to_blocks_spec)
S3method(blocks_to_rowrecs,default)
S3method(blocks_to_rowrecs,relop)
S3method(blocks_to_rowrecs,wrapped_relop)
S3method(build_pivot_control,default)
S3method(build_pivot_control,relop)
S3method(build_pivot_control,wrapped_relop)
S3method(format,blocks_to_rowrecs_spec)
S3method(format,cdata_general_transform_spec)
S3method(format,rowrecs_to_blocks_spec)
S3method(layout_by,blocks_to_rowrecs_spec)
S3method(layout_by,cdata_general_transform_spec)
S3method(layout_by,rowrecs_to_blocks_spec)
S3method(print,blocks_to_rowrecs_spec)
S3method(print,cdata_general_transform_spec)
S3method(print,rowrecs_to_blocks_spec)
S3method(rowrecs_to_blocks,default)
S3method(rowrecs_to_blocks,relop)
S3method(t,blocks_to_rowrecs_spec)
S3method(t,cdata_general_transform_spec)
S3method(t,rowrecs_to_blocks_spec)
S3method(unpivot_to_blocks,default)
S3method(unpivot_to_blocks,relop)
S3method(unpivot_to_blocks,wrapped_relop)
export("%**%")
export("%//%")
export(blocks_to_rowrecs)
export(blocks_to_rowrecs_q)
export(blocks_to_rowrecs_spec)
export(build_pivot_control)
export(build_pivot_control_q)
export(build_unpivot_control)
export(check_cols_form_unique_keys)
export(convert_cdata_spec_to_yaml)
export(convert_records)
export(convert_yaml_to_cdata_spec)
export(get_transform_details)
export(layout_by)
export(layout_specification)
export(layout_to_blocks)
export(layout_to_rowrecs)
export(map_fields)
export(map_fields_q)
export(pivot_to_blocks)
export(pivot_to_rowrecs)
export(qlook)
export(rowrecs_to_blocks)
export(rowrecs_to_blocks_q)
export(rowrecs_to_blocks_spec)
export(rows_are_uniquely_keyed)
export(unpivot_to_blocks)
importFrom(methods,is)
importFrom(methods,setGeneric)
importFrom(rqdatatable,ex_data_table)
importFrom(stats,as.formula)
importFrom(stats,complete.cases)
importFrom(wrapr,"%.>%")
importFrom(wrapr,apply_left)
importFrom(wrapr,apply_right)
importFrom(wrapr,let)
importFrom(wrapr,mk_tmp_name_source)
